# -*- coding: utf-8 -*-
#
#  OX for UCS
#  listener module: handling fetchmail accounts
#
# Copyright (C) 2004-2019 Univention GmbH
#
# http://www.univention.de/
#
# and
#
# iKu Systems & Services GmbH & Co. KG
# http://www.iku-systems.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

__package__ = '' 	# workaround for PEP 366
import os
import traceback
try:
	from typing import Optional, Union
except ImportError:
	pass

from ldap.filter import filter_format
import listener
import univention.uldap
import univention.debug


name = 'ox-fetchmail'
description = 'Update Open-Xchange fetchmailrc'
filter = '(&(|(objectClass=oxFetchmailMultiDrop)(objectClass=oxFetchmailSingleDrop))(cn=*))'
attributes = []

ldapbase = listener.baseConfig['ldap/base']

fetchmailrc = '/etc/fetchmailrc'
default_context = int(listener.configRegistry.get('ox/context/id', 10))


def initialize():
	pass


def handler(dn, new, old):
	fetchmailrcline1 = ''
	fetchmailrcline2 = ''
	fetchmailrcline3 = ''
	fetchmailrcline4 = ''

	listener.setuid(0)
	try:
		ldapCon = univention.uldap.getMachineConnection(ldap_master=False)

		deletedcn = ''
		if old and 'cn' in old and not new:
			deletedcn = old['cn']

		f = open(fetchmailrc, 'w')

		contextadmin = get_context_admin_user(default_context)

		f.write('# Do not edit this file!\n# This file will by overwritten by Univention Directory Manager\nset syslog\nset daemon 300\n')
		results = ldapCon.search(base="cn=users,%s" % ldapbase, filter='uid=%s' % (contextadmin,), attr=['mailPrimaryAddress'])
		if results:
			_, attrs = results[0]
			adminmail = attrs['mailPrimaryAddress'][0]
			f.write('set postmaster "%s"\n\n' % (adminmail,))
		results = ldapCon.search(base="cn=mail,%s" % ldapbase, filter=filter, attr=['*'])
		for dn, attrs in results:
			thiscn = dn
			try:
				log('ox-fetchmail: dn: %r attrs: %r' % (dn, attrs))
				if attrs and attrs.get('cn'):
					thiscn = attrs['cn']
					log('ox-fetchmail: thiscn: %s' % (thiscn,))
				if attrs and attrs.get('cn'):
					cn = attrs['cn'][0]
					log('ox-fetchmail: cn: %s' % (cn,))
					fetchmailrcline1 = ('# identifier %s' % (cn,))

				# the next four blocks are useless, as fetchmailrcline2 will be overwritten anyway
				if attrs and attrs.get('remoteServer'):
					remoteServer = attrs['remoteServer'][0]
					log('ox-fetchmail: remoteServer: %s' % (remoteServer,))
					fetchmailrcline2 = ('poll \'%s\'' % (remoteServer,))

				if attrs and attrs.get('protocol'):
					protocol = attrs['protocol'][0]
					log('ox-fetchmail: protocol: %s' % (protocol,))
					fetchmailrcline2 = ('%s with proto %s' % (fetchmailrcline2, protocol))

				if attrs and attrs.get('envelopeHeader'):
					envelopeHeader = attrs['envelopeHeader'][0]
					log('ox-fetchmail: envelopeHeader: %s' % (envelopeHeader,))
					fetchmailrcline2 = ('%s envelope %s' % (fetchmailrcline2, envelopeHeader))

				if attrs and attrs.get('qmailPrefix'):
					qmailPrefix = attrs['qmailPrefix'][0]
					log('ox-fetchmail: qmailPrefix: %s' % (qmailPrefix,))
					fetchmailrcline2 = ('%s qvirtual %s' % (fetchmailrcline2, qmailPrefix))
				# this overwrites the previous four blocks - does this make any sense?
				fetchmailrcline2 = ('%s no dns' % (fetchmailrcline2,))

				localDomainName = ''
				if attrs and attrs.get('localDomainName'):
					if attrs['localDomainName']:
						localDomainName = attrs['localDomainName'][0]
					log('ox-fetchmail: localDomainName: %s' % (localDomainName,))
				if attrs and 'localRecipient' not in attrs:
					if not localDomainName:
						localDomainName = listener.baseConfig['mail/hosteddomains']
					fetchmailrcline3 = ('	localdomains %s:' % (localDomainName,))

				if attrs and attrs.get('remoteUser'):
					remoteUser = attrs['remoteUser'][0]
					log('ox-fetchmail: remoteUser: %s' % (remoteUser,))
					fetchmailrcline4 = ('	user \'%s\' there' % (remoteUser,))

				if attrs and attrs.get('remotePassword'):
					remotePassword = attrs['remotePassword'][0]
					# log('ox-fetchmail: remotePassword: %s' % remotePassword, level=univention.debug.ERROR)
					fetchmailrcline4 = ('%s with password \'%s\'' % (fetchmailrcline4, remotePassword))

				if attrs and attrs.get('localRecipient'):
					localRecipient = attrs['localRecipient'][0]
					log('ox-fetchmail: localRecipient: %s' % (localRecipient,))
					uid_result_id = ldapCon.search(base=ldapbase, filter=filter_format('uidNumber=%s', (localRecipient,)), attr=['mailPrimaryAddress'])
					localRecipient = uid_result_id[0][1]['mailPrimaryAddress'][0]
					fetchmailrcline4 = ('%s is \'%s\' here' % (fetchmailrcline4, localRecipient))
				else:
					fetchmailrcline4 = ('%s is * here' % (fetchmailrcline4,))

				if attrs and attrs.get('leaveMessagesOnServer'):
					leaveMessagesOnServer = attrs['leaveMessagesOnServer'][0]
					log('ox-fetchmail: leaveMessagesOnServer: %s' % (leaveMessagesOnServer,))
					if leaveMessagesOnServer == 'OK':
						fetchmailrcline4 += ' keep'
				if attrs and attrs.get('useSSL'):
					useSSL = attrs['useSSL'][0]
					log('ox-fetchmail: useSSL: %s' % (useSSL,))
					if useSSL == 'OK':
						fetchmailrcline4 += ' ssl'

				log('ox-fetchmail: oldcn: %s, thiscn: %s' % (deletedcn, thiscn))
				# if not thiscn[0] == deletedcn[0]:
				if True:
					# log('ox-fetchmail: line1: %s' % fetchmailrcline1, level=univention.debug.ERROR)
					f.write('%s\n' % (fetchmailrcline1,))
					# log('ox-fetchmail: line2: %s' % fetchmailrcline2, level=univention.debug.ERROR)
					f.write('%s\n' % (fetchmailrcline2,))
					if localDomainName:
						# log('ox-fetchmail: line3: %s' % fetchmailrcline3, level=univention.debug.ERROR)
						f.write('%s\n' % (fetchmailrcline3,))
					# log('ox-fetchmail: line4: %s' % fetchmailrcline4, level=univention.debug.ERROR)
					f.write('%s\n\n' % (fetchmailrcline4,))
			except Exception:
				log(traceback.format_exc(), level=univention.debug.ERROR)
			finally:
				log('ox-fetchmail: ID %s done (DN %r).' % (thiscn, dn))

		f.close()
		os.chmod(fetchmailrc, 384)
		os.system('chown %s:%s %s' % ('fetchmail', 'root', fetchmailrc))
	finally:
		listener.unsetuid()


def postrun():
	listener.setuid(0)
	try:
		log('ox-fetchmail: restarting fetchmail ...')
		os.spawnv(os.P_WAIT, '/etc/init.d/fetchmail', ['fetchmail', 'force-reload'])
	finally:
		listener.unsetuid()


# copied from univention.ox.listener_tools module to remove circular package dependency
def get_context_admin_user(context_id):  # type: (Union[int, str]) -> str
	if int(context_id) == 10:
		return 'oxadmin'
	else:
		return 'oxadmin-context{}'.format(context_id)


def log(msg, level=univention.debug.INFO):  # type: (str, Optional[int]) -> None
	univention.debug.debug(univention.debug.LISTENER, level, msg)
